defmodule STBU.ClinicTest do
  use ExUnit.Case

  alias STBU.Interface.Clinic
  alias STBU.Interface.AppointmentScheduler, as: AS

  @test0 "0-in.json"
  @timeout 50

  setup_all do
    {:ok, in_dev} = File.open(@test0, [:read, :utf8])
    {shelter, clinic} = STBU.Interface.Entry.initialize_shelter_and_clinic(in_dev, @timeout)

    on_exit(fn ->
      Process.exit(shelter, :kill)
      Process.exit(clinic, :kill)
    end)

    :ok = File.close(in_dev)
    %{clinic: clinic}
  end

  test "schedule an appointment", %{clinic: clinic} do
    animal = %{name: "Fido", age: 30, type: :dog, beenVaccinated: true, picture: ""}
    scheduler = Clinic.request_appointment(clinic, animal)
    assert(is_pid(scheduler))

    _appt = select_randomly(scheduler)

    assert AS.finalize_details(scheduler) == :ok
    assert AS.book(scheduler) == :ok
  end

  test "reservation timeout", %{clinic: clinic} do
    animal = %{name: "Fido", age: 30, type: :dog, beenVaccinated: true, picture: ""}
    scheduler = Clinic.request_appointment(clinic, animal)
    assert(is_pid(scheduler))

    appt = select_randomly(scheduler)

    assert AS.finalize_details(scheduler) == :ok
    assert_receive({:appointment_timeout, ^appt}, @timeout + 10)
  end

  def select_randomly(scheduler) do
    vets = AS.available_vets(scheduler)
    AS.select_vet(scheduler, Enum.random(vets))
    weeks = AS.available_weeks(scheduler)
    AS.select_week(scheduler, Enum.random(weeks))
    days = AS.available_days(scheduler)
    AS.select_day(scheduler, Enum.random(days))
    times = AS.available_times(scheduler)
    AS.select_time(scheduler, Enum.random(times))
    AS.current_selection(scheduler)
  end
end
