defmodule STBU.HandoffTest do
  use ExUnit.Case

  alias STBU.Interface.{Shelter, PetViewer}
  alias STBU.Interface.AppointmentScheduler, as: AS

  @test0 "0-in.json"
  @timeout 50

  setup_all do
    {:ok, in_dev} = File.open(@test0, [:read, :utf8])
    {shelter, clinic} = STBU.Interface.Entry.initialize_shelter_and_clinic(in_dev, @timeout)

    on_exit(fn ->
      Process.exit(shelter, :kill)
      Process.exit(clinic, :kill)
    end)

    :ok = File.close(in_dev)
    %{shelter: shelter, clinic: clinic}
  end

  test "adopt unvaccinated then schedule", %{shelter: shelter, clinic: _clinic} do
    viewer = Shelter.view_pets(shelter)
    animal = first_unvaccinated(viewer)
    {:schedule, scheduler} = PetViewer.try_adopt(viewer)
    assert is_pid(scheduler)
    _appt = STBU.ClinicTest.select_randomly(scheduler)
    assert AS.finalize_details(scheduler) == :ok
    assert AS.book(scheduler) == :ok
    assert_receive({:adopted, ^animal}, 5)
  end

  test "handoff timeout", %{shelter: shelter, clinic: _clinic} do
    viewer = Shelter.view_pets(shelter)
    animal = first_unvaccinated(viewer)
    {:schedule, scheduler} = PetViewer.try_adopt(viewer)
    assert is_pid(scheduler)
    _appt = STBU.ClinicTest.select_randomly(scheduler)
    assert_receive({:adoption_failed, ^animal}, @timeout + 10)
  end


  def first_unvaccinated(viewer) do
    animal = PetViewer.view_next(viewer)

    if animal && !animal.beenVaccinated do
      animal
    else
      first_unvaccinated(viewer)
    end
  end
end
