defmodule STBU.ShelterTests do
  use ExUnit.Case

  alias STBU.Interface.{Shelter, PetViewer}

  @test0 "0-in.json"
  @timeout 50

  setup_all do
    {:ok, in_dev} = File.open(@test0, [:read, :utf8])
    {shelter, clinic} = STBU.Interface.Entry.initialize_shelter_and_clinic(in_dev, @timeout)
    :ok = File.close(in_dev)

    on_exit(fn ->
      Process.exit(shelter, :kill)
      Process.exit(clinic, :kill)
    end)

    %{shelter: shelter}
  end

  test "adopt a pet", %{shelter: shelter} do
    assert is_pid(shelter)
    viewer = Shelter.view_pets(shelter)
    _animal = first_vaccinated(viewer)
    assert(PetViewer.try_play(viewer) == :ok)
    assert(PetViewer.try_adopt(viewer) == :ok)
  end

  def first_vaccinated(viewer) do
    animal = PetViewer.view_next(viewer)

    if animal && animal.beenVaccinated do
      animal
    else
      first_vaccinated(viewer)
    end
  end

  test "play timeout", %{shelter: shelter} do
    viewer = Shelter.view_pets(shelter)
    animal = first_vaccinated(viewer)
    assert(PetViewer.try_play(viewer) == :ok)
    assert_receive({:play_timeout, ^animal}, @timeout + 10)
  end
end
